@@ -13,15 +13,24 @@ from datetime import timedelta
 from datetime import timezone
 from inspect import getfullargspec
 from inspect import getmembers
-from typing import Any, Callable, Dict, Generic, List, Mapping, Optional, TypeVar, Type, Union, Set, overload
+from typing import Any, Callable, Dict, Generic, List, Mapping, Optional, TypeVar, Type, Union, Set, overload, Iterable
 from typing import TYPE_CHECKING
 
 from pynamodb._compat import GenericMeta
-from pynamodb.constants import (
-    BINARY, BINARY_SET, BOOLEAN, DATETIME_FORMAT, DEFAULT_ENCODING,
-    LIST, MAP, NULL, NUMBER, NUMBER_SET, STRING, STRING_SET
-)
-from pynamodb.exceptions import AttributeDeserializationError, AttributeNullError
+from pynamodb.constants import BINARY
+from pynamodb.constants import BINARY_SET
+from pynamodb.constants import BOOLEAN
+from pynamodb.constants import DATETIME_FORMAT
+from pynamodb.constants import DEFAULT_ENCODING
+from pynamodb.constants import LIST
+from pynamodb.constants import MAP
+from pynamodb.constants import NULL
+from pynamodb.constants import NUMBER
+from pynamodb.constants import NUMBER_SET
+from pynamodb.constants import STRING
+from pynamodb.constants import STRING_SET
+from pynamodb.exceptions import AttributeDeserializationError
+from pynamodb.exceptions import AttributeNullError
 from pynamodb.expressions.operand import Path
 
 
@@ -73,7 +82,7 @@ class Attribute(Generic[_T]):
         self.is_hash_key = hash_key
         self.is_range_key = range_key
 
-        # AttributeContainerMeta._initialize_attributes will ensure this is a string
+        # __set_name__ will ensure this is a string
         self.attr_path: List[str] = [attr_name]  # type: ignore
 
     @property
@@ -106,6 +115,9 @@ class Attribute(Generic[_T]):
         else:
             return self
 
+    def __set_name__(self, owner: Type[Any], name: str) -> None:
+        self.attr_name = self.attr_name or name
+
     def _is_map_attribute_class_object(self, instance: 'Attribute') -> bool:
         return isinstance(instance, MapAttribute) and not instance._is_attribute_container()
 
@@ -198,10 +210,10 @@ class Attribute(Generic[_T]):
     def __or__(self, other: Any) -> '_IfNotExists':
         return Path(self).__or__(other)
 
-    def append(self, other: Any) -> '_ListAppend':
+    def append(self, other: Iterable) -> '_ListAppend':
         return Path(self).append(other)
 
-    def prepend(self, other: Any) -> '_ListAppend':
+    def prepend(self, other: Iterable) -> '_ListAppend':
         return Path(self).prepend(other)
 
     def set(self, value: Any) -> 'SetAction':
@@ -236,24 +248,10 @@ class AttributeContainerMeta(GenericMeta):
         cls._dynamo_to_python_attrs = {}
 
         for name, attribute in getmembers(cls, lambda o: isinstance(o, Attribute)):
-            initialized = False
-            if isinstance(attribute, MapAttribute):
-                # MapAttribute instances that are class attributes of an AttributeContainer class
-                # should behave like an Attribute instance and not an AttributeContainer instance.
-                initialized = attribute._make_attribute()
-
             cls._attributes[name] = attribute
-            if attribute.attr_name is None:
-                attribute.attr_name = name
             if attribute.attr_name != name:
                 cls._dynamo_to_python_attrs[attribute.attr_name] = name
 
-            if initialized and isinstance(attribute, MapAttribute):
-                # To support creating expressions from nested attributes, MapAttribute instances
-                # store local copies of the attributes in cls._attributes with `attr_path` set.
-                # Prepend the `attr_path` lists with the dynamo attribute name.
-                attribute._update_attribute_paths(attribute.attr_name)
-
         # Register the class with the discriminator if necessary.
         discriminators = [name for name, attr in cls._attributes.items() if isinstance(attr, DiscriminatorAttribute)]
         if len(discriminators) > 1:
@@ -380,6 +378,36 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
                 value = attr.deserialize(attr.get_value(attribute_value))
                 setattr(self, name, value)
 
+    @classmethod
+    def _update_attribute_types(cls, attribute_values: Dict[str, Dict[str, Any]]):
+        """
+        Update the attribute types in the attribute values dictionary to disambiguate json string and array types
+        """
+        for attr in cls.get_attributes().values():
+            attribute_value = attribute_values.get(attr.attr_name)
+            if attribute_value:
+                AttributeContainer._coerce_attribute_type(attr.attr_type, attribute_value)
+                if isinstance(attr, ListAttribute) and attr.element_type and LIST in attribute_value:
+                    if issubclass(attr.element_type, AttributeContainer):
+                        for element in attribute_value[LIST]:
+                            if MAP in element:
+                                attr.element_type._update_attribute_types(element[MAP])
+                    else:
+                        for element in attribute_value[LIST]:
+                            AttributeContainer._coerce_attribute_type(attr.element_type.attr_type, element)
+                if isinstance(attr, AttributeContainer) and MAP in attribute_value:
+                    attr._update_attribute_types(attribute_value[MAP])
+
+    @staticmethod
+    def _coerce_attribute_type(attr_type: str, attribute_value: Dict[str, Any]):
+        # coerce attribute types to disambiguate json string and array types
+        if attr_type == BINARY and STRING in attribute_value:
+            attribute_value[BINARY] = attribute_value.pop(STRING)
+        if attr_type in {BINARY_SET, NUMBER_SET, STRING_SET} and LIST in attribute_value:
+            json_type = NUMBER if attr_type == NUMBER_SET else STRING
+            if all(next(iter(v)) == json_type for v in attribute_value[LIST]):
+                attribute_value[attr_type] = [value[json_type] for value in attribute_value.pop(LIST)]
+
     @classmethod
     def _get_discriminator_class(cls, attribute_values: Dict[str, Dict[str, Any]]) -> Optional[Type]:
         discriminator_attr = cls._get_discriminator_attribute()
@@ -779,7 +807,7 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
     does not behave as both an Attribute AND an AttributeContainer. Rather an instance of MapAttribute behaves
     EITHER as an Attribute OR as an AttributeContainer, depending on where it was instantiated.
 
-    So, how do we create this dichotomous behavior? Using the AttributeContainerMeta metaclass.
+    So, how do we create this dichotomous behavior?
     All MapAttribute instances are initialized as AttributeContainers only. During construction of
     AttributeContainer classes (subclasses of MapAttribute and Model), any instances that are class attributes
     are transformed from AttributeContainers to Attributes (via the `_make_attribute` method call).
@@ -793,7 +821,7 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         self.attribute_kwargs = {arg: attributes.pop(arg) for arg in self.attribute_args if arg in attributes}
 
         # Assume all instances should behave like an AttributeContainer. Instances that are intended to be
-        # used as Attributes will be transformed by AttributeContainerMeta during creation of the containing class.
+        # used as Attributes will be transformed during creation of the containing class.
         # Because of this do not use MRO or cooperative multiple inheritance, call the parent class directly.
         AttributeContainer.__init__(self, **attributes)
 
@@ -814,10 +842,9 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         return 'attribute_values' in self.__dict__
 
     def _make_attribute(self):
-        # WARNING! This function is only intended to be called from the AttributeContainerMeta metaclass.
+        # WARNING! This function is only intended to be called from the __set_name__ function.
         if not self._is_attribute_container():
-            # This instance has already been initialized by another AttributeContainer class.
-            return False
+            raise AssertionError("MapAttribute._make_attribute called on an initialized instance")
         # During initialization the kwargs were stored in `attribute_kwargs`. Remove them and re-initialize the class.
         kwargs = self.attribute_kwargs
         del self.attribute_kwargs
@@ -829,10 +856,9 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
             # we have to store the local copy directly into __dict__ to prevent calling attr.__set__.
             # Use deepcopy so that `attr_path` and any local attributes are also copied.
             self.__dict__[name] = deepcopy(attr)
-        return True
 
     def _update_attribute_paths(self, path_segment):
-        # WARNING! This function is only intended to be called from the AttributeContainerMeta metaclass.
+        # WARNING! This function is only intended to be called from the __set_name__ function.
         if self._is_attribute_container():
             raise AssertionError("MapAttribute._update_attribute_paths called before MapAttribute._make_attribute")
         for name in self.get_attributes().keys():
@@ -910,6 +936,19 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
             value = type(self)(**value)  # type: ignore
         return super().__set__(instance, value)  # type: ignore
 
+    def __set_name__(self, owner: Type[Any], name: str) -> None:
+        if issubclass(owner, AttributeContainer):
+            # MapAttribute instances that are class attributes of an AttributeContainer class
+            # should behave like an Attribute instance and not an AttributeContainer instance.
+            self._make_attribute()
+
+            super().__set_name__(owner, name)
+
+            # To support creating expressions from nested attributes, MapAttribute instances
+            # store local copies of the attributes in cls._attributes with `attr_path` set.
+            # Prepend the `attr_path` lists with the dynamo attribute name.
+            self._update_attribute_paths(self.attr_name)
+
     def _set_attributes(self, **attrs):
         """
         Sets the attributes for this object
@@ -1174,4 +1213,5 @@ SERIALIZE_CLASS_MAP = {
     float: NumberAttribute(),
     int: NumberAttribute(),
     str: UnicodeAttribute(),
+    bytes: BinaryAttribute(),
 }
